#encoding:utf-8

class Router(object):
    
    def compare(self,chunks,comparator):
        """
        chunk选择器
        """
        i = 1
        for j in range(1, len(chunks)):
            rlt = comparator(chunks[j], chunks[0])
            if rlt > 0:
                i = 0
            if rlt >= 0:
                chunks[i], chunks[j] = chunks[j], chunks[i]
                i += 1
        return chunks[0:i]
    

    def __call__(self,chunks):
        """
        选择chunks
        """
        chunks = self.maximum_matching(chunks)
        if len(chunks) > 1:
            chunks = self.largest_average(chunks)
        if len(chunks) > 1:
            chunks = self.smallest_variance(chunks)
        if len(chunks) > 1:
            chunks = self.morphemic_freedom(chunks)
        return chunks

    def maximum_matching(self,chunks):
        """
        最大匹配
        """
        def comparator(a,b):
            return a.total_word_length() - b.total_word_length()
        return self.compare(chunks,comparator)

    def largest_average(self,chunks):
        """
        单词的最大平均长度
        """
        def comparator(a,b):
            return a.average_word_length() - b.average_word_length()
        return self.compare(chunks,comparator)

    def smallest_variance(self,chunks):
        """
        词语长度的最小变化率
        """
        def comparator(a,b):
            return b.standard_deviation() - a.standard_deviation()
        return self.compare(chunks, comparator)
    
    def morphemic_freedom(self,chunks):
        """
        计算词组中的所有单字词词频的自然对数,
        然后将得到的值相加，取总和最大的词组。
        """
        
        def comparator(a,b):
            return a.word_frequency() - b.word_frequency()
        return self.compare(chunks, comparator)

route = Router()



#encoding:utf-8
#词典
import os
from scseg.digital import is_chinese_number

#单个词实体映射
class Word(object):

    def __init__(self,text = '',freq = 0,length = -1):
        self.text = text#词
        self.freq = freq#词频
        self.length = len(text) if length<0 else length#有效词长

    def __str__(self):
        return self.text
    
    def __repr__(self):
        return self.text

    def __len__(self):
        return self.length

class Dictionary(object):

    dict_words = {}

    def __init__(self,root_path):
        #载入字库
        if not Dictionary.dict_words:
            for path in os.listdir(root_path): 
                if path.endswith('.dic'):
                    self.load(os.sep.join([root_path,path]))

    @staticmethod
    def load(path):
        """
        载入字典
        """
        for line in open(path):
            words = line.split(' ')
            if len(words)==2:
                word = unicode(words[0].strip(), 'utf-8')
                Dictionary.dict_words[word] = Word(word, int(words[1]))
            elif len(words)==1:
                word = unicode(words[0].strip(),'utf-8')
                Dictionary.dict_words[word] = Word(word,0)

    def __len__(self):
        return len(self.dict_words)

    def __getitem__(self,word):
        if Dictionary.dict_words.has_key(word):
            return Dictionary.dict_words[word]
        elif is_chinese_number(word):#数字识别
            return Word(word,0)
        elif len(word) == 1:#生僻字词频为0
            return Word(word,0)
        else:
            return None




#encoding:utf-8

import chardet
from .core import Splitter,KeywordSplitter
from .route.mmseg import route

def seg_text(text):
    if not isinstance(text,unicode):
        encoding = chardet.detect(text)['encoding']
        text = unicode(text,encoding)
    return [word for word in Splitter(text)]

def keywords(text):
    if not isinstance(text,unicode):
        encoding = chardet.detect(text)['encoding']
        text = unicode(text,encoding)
    return [word for word in KeywordSplitter(text)]


#encoding:utf-8
import os
import re
from route.mmseg import route
from .word import Word,Dictionary
from .digital import is_chinese_number,chinese_to_number

here = os.path.abspath(os.path.dirname(__file__))
dict_words = Dictionary(here+os.sep+'data')


class Chunk(object):

    def __init__(self,*words):
        self.words = []
        for word in words:
            if len(word) == 0:
                continue
            self.words.append(word)
    
	#计算chunk的总长度
    def total_word_length(self):
        length = 0
        for word in self.words:
            length += len(word)
        return length
    
	#计算平均长度
    def average_word_length(self):
        return float(self.total_word_length()) / float(len(self.words))

    #统计词频
    def word_frequency(self):
        sum = 0
        for word in self.words:
            sum += word.freq
        return sum
    
	#计算标准差
    def standard_deviation(self):
        average = self.average_word_length()
        sum = 0.0
        for word in self.words:
            tmp = (len(word) - average)
            sum += float(tmp) * float(tmp)
        return sum



class BaseSplitter(object):
    
    def __init__(self,text): 
        self.text = text  
        self.pos = 0
        self.text_length = len(self.text)  
    
    def next_char(self):  
        return self.text[self.pos]  

    #判断该字符是否是中文字符（不包括中文标点）    
    def is_cjk_char(self,charater):  
        c = ord(charater)
        return 0x4E00<= c <=0x9FFF or\
               0x3400<= c <=0x4dbf or\
               0xf900<= c <=0xfaff or\
               0x3040<= c <=0x309f or\
               0xac00<= c <=0xd7af
    
    #判断是否是ASCII码  
    def is_latin_char(self, ch):  
        import string  
        if ch in string.whitespace:  
            return False  
        if ch in string.punctuation:  
            return False  
        return ch in string.printable  

    #切割出非中文词  
    def get_latin_words(self):  
        # Skip pre-word whitespaces and punctuations  
        #跳过中英文标点和空格  
        while self.pos < self.text_length:  
            ch = self.next_char()  
            if self.is_latin_char(ch) or self.is_cjk_char(ch):  
                break  
            self.pos += 1  
        #得到英文单词的起始位置      
        start = self.pos  
          
        #找出英文单词的结束位置  
        while self.pos < self.text_length:  
            ch = self.next_char()  
            if not self.is_latin_char(ch):  
                break  
            self.pos += 1  
        end = self.pos  
          
        #Skip chinese word whitespaces and punctuations  
        #跳过中英文标点和空格  
        while self.pos < self.text_length:  
            ch = self.next_char()  
            if self.is_latin_char(ch) or self.is_cjk_char(ch):  
                break  
            self.pos += 1  
              
        #返回英文单词  
        return Word(self.text[start:end])

    #运用正向最大匹配算法结合字典来切割中文文本    
    def get_match_cjk_words(self):  
        originalPos = self.pos  
        words = []  
        index = 0  
        while self.pos < self.text_length:  
            if index >= len(dict_words) :  
                break  
            if not self.is_cjk_char(self.next_char()):
                break  
            self.pos += 1  
            index += 1  
              
            text = self.text[originalPos:self.pos]  
            word = dict_words[text]  
            if word:  
                words.append(word)  
                  
        self.pos = originalPos  
        if not words:words.append(Word('X',0,0))#添加结束词 
        return words


class Splitter(BaseSplitter):  
      
    def __init__(self,text,route=route): 
        BaseSplitter.__init__(self, text)  
        self.route = route  
              
      
    #得到下一个切割结果  
    def __iter__(self):  
        while self.pos < self.text_length:  
            if self.is_cjk_char(self.next_char()):  
                for word in self.get_cjk_words():
                    if len(word) > 0:
                        word = unicode(word)
                        yield word
            else :  
                word = self.get_latin_words() 
                if len(word) > 0:  
                    yield unicode(word)  
        raise StopIteration
      
    #切割出中文词，并且做处理，用上述4种方法  
    def get_cjk_words(self):  
        #应用规则过滤
        chunks = self.route(self.create_chunks())
          
        #最后只有一种切割方法  
        chunk = chunks[0] 
        self.pos += chunk.total_word_length()  
        return chunk.words
      
    #三重循环来枚举切割方法，这里也可以运用递归来实现  
    def create_chunks(self):  
        chunks = []  
        originalPos = self.pos  
        words1 = self.get_match_cjk_words()  
          
        for word1 in words1:  
            self.pos += len(word1)  
            if self.pos < self.text_length:  
                words2 = self.get_match_cjk_words()  
                for word2 in words2:  
                    self.pos += len(word2)  
                    if self.pos < self.text_length:  
                        words3 = self.get_match_cjk_words()  
                        for word3 in words3:  
                            chunk = Chunk(word1,word2,word3)  
                            chunks.append(chunk)  
                    elif self.pos == self.text_length: 
                        chunks.append(Chunk(word1,word2))  
                    self.pos -= len(word2)  
            elif self.pos == self.text_length:
                chunks.append(Chunk(word1))  
            self.pos -= len(word1)  
        self.pos = originalPos
        return chunks


class KeywordSplitter(BaseSplitter):  
      
    def __init__(self,text,route=route): 
        BaseSplitter.__init__(self, text)  
        self.route = route  
              
      
    #得到下一个切割结果  
    def __iter__(self):  
        result = set()
        while self.pos < self.text_length:  
            if self.is_cjk_char(self.next_char()):
                for words in self.get_cjk_words():
                    for word in words:
                        if len(word) > 0:
                            word = unicode(word)
                            result.add(word)
            else :  
                word = self.get_latin_words() 
                if len(word) > 0:  
                    result.add(word)
        for word in result:
            yield unicode(word)
        raise StopIteration
      
    #切割出中文词，并且做处理，用上述4种方法  
    def get_cjk_words(self):  
        #应用规则过滤
        chunks = self.create_chunks()
          
        #最后只有一种切割方法  
        for chunk in chunks:
            self.pos += chunk.total_word_length()  
            yield chunk.words
      
    #三重循环来枚举切割方法，这里也可以运用递归来实现  
    def create_chunks(self):  
        chunks = []  
        originalPos = self.pos  
        words1 = self.get_match_cjk_words()  
          
        for word1 in words1:  
            self.pos += len(word1)  
            if self.pos < self.text_length:  
                words2 = self.get_match_cjk_words()  
                for word2 in words2:  
                    self.pos += len(word2)  
                    if self.pos < self.text_length:  
                        words3 = self.get_match_cjk_words()  
                        for word3 in words3:  
                            chunk = Chunk(word1,word2,word3)  
                            chunks.append(chunk)  
                    elif self.pos == self.text_length: 
                        chunks.append(Chunk(word1,word2))  
                    self.pos -= len(word2)  
            elif self.pos == self.text_length:
                chunks.append(Chunk(word1))  
            self.pos -= len(word1)  
        self.pos = originalPos
        return chunks



#encoding:utf-8

from scseg import seg_text
from scseg.word import Dictionary


class Learning(object):
    
    def __init__(self,times=3,train = None):
        self.cache = {}
        self.times = times
        if train:
            del Dictionary.dict_words 
            Dictionary.dict_words= {}
            Dictionary.load(train)

    def learn(self,path):
        """
        待学习的文件路径
        """
        count = 0
        for line in open(path):
            count = count +1
            line = unicode(line,'utf-8')
            tmp = []
            for word in seg_text(line):
                if len(word) == 1:
                    tmp.append(word)
                else:
                    if len(tmp)>1:
                        new_word=''.join(tmp)
                        if new_word in self.cache:
                            self.cache[new_word] = self.cache[new_word]+1
                        else:
                            self.cache[new_word] = 1
                    del tmp
                    tmp = []
            if count%1000 == 0:
                print("count:%d" % count)
    def save(self,path):
        with open(path,'w') as f:
            for word in self.cache.viewitems():
                if word[1]>=self.times:
                    f.write(word[0].encode("utf-8")+'\n')



#encoding:utf-8
import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
# need install xapian-binding-python
scripts = ['scripts/learn']
requires = [
    'nose',
    'chardet',
    ]

setup(name='scseg',
      version='1.4.6',
      description='a python chinese seg word',
      long_description=README + '\n\n' +  CHANGES,
      classifiers=[
        "Programming Language :: Python",
        ],
      author='duanhongyi',
      author_email='duanhyi@gmail.com',
      url='https://bitbucket.org/duanhongyi/scseg',
      keywords='Chinese Word Segmentation 中文 分词 工具包 汉语',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=requires,
      scripts = scripts,
      platforms = 'all platform',
      license = 'BSD',
      )




#encoding:utf-8
import re

CN_NUM = {
u'〇' : 0,
u'一' : 1,
u'二' : 2,
u'三' : 3,
u'四' : 4,
u'五' : 5,
u'六' : 6,
u'七' : 7,
u'八' : 8,
u'九' : 9,

u'零' : 0,
u'壹' : 1,
u'贰' : 2,
u'叁' : 3,
u'肆' : 4,
u'伍' : 5,
u'陆' : 6,
u'柒' : 7,
u'捌' : 8,
u'玖' : 9,

u'貮' : 2,
u'两' : 2,
}
CN_UNIT = {
u'十' : 10,
u'拾' : 10,
u'百' : 100,
u'佰' : 100,
u'千' : 1000,
u'仟' : 1000,
u'万' : 10000,
u'萬' : 10000,
u'亿' : 100000000,
u'億' : 100000000,
u'兆' : 1000000000000,
}


has_unit  = re.compile((u'+[*?]*|'.join(CN_UNIT.keys())), re.UNICODE).findall

chinese_number_regex =  '^['+u''.join(CN_UNIT.keys()+CN_NUM.keys())+']+$'
is_chinese_number = re.compile(chinese_number_regex,re.UNICODE).match


def chinese_to_number(cn):
    if has_unit(cn):
        lcn = list(cn)
        unit = 0 #当前的单位
        ldig = []#临时数组
        while lcn:
            cndig = lcn.pop()
            if CN_UNIT.has_key(cndig):
                unit = CN_UNIT.get(cndig)
                if unit==10000:
                    ldig.append('w')    #标示万位
                    unit = 1
                elif unit==100000000:
                    ldig.append('y')    #标示亿位
                    unit = 1
                elif unit==1000000000000:#标示兆位
                    ldig.append('z')
                    unit = 1
                continue
            else:
                dig = CN_NUM.get(cndig)
                if unit:
                    dig = dig*unit
                    unit = 0
                ldig.append(dig)
        if unit==10:    #处理10-19的数字
            ldig.append(10)
        ret = 0
        tmp = 0
        while ldig:
            x = ldig.pop()
            if x=='w':
                tmp *= 10000
                ret += tmp
                tmp=0
            elif x=='y':
                tmp *= 100000000
                ret += tmp
                tmp=0
            elif x=='z':
                tmp *= 1000000000000
                ret += tmp
                tmp=0
            else:
                tmp += x
        ret += tmp
        return ret
    else:
        return int(''.join([str(CN_NUM[c]) for c in cn]))
